#!/usr/bin/env python3

from itertools import product
import csv
import gurobipy as gp
import json
import sys


def solve(data, alpha):
    n = len(data["cost"])
    workers = range(n)
    patients = range(n)

    m = gp.Model()
    m.ModelSense = gp.GRB.MINIMIZE
    m.Params.OutputFlag = 0

    # x[w,p] = 1 if worker w is assigned to patient p.
    x = m.addVars(n, n, vtype=gp.GRB.BINARY)

    for i in range(n):
        # Each worker is assigned to one patient.
        m.addConstr(gp.quicksum(x[i, p] for p in patients) == 1)

        # Each patient is assigned one worker.
        m.addConstr(gp.quicksum(x[w, i] for w in workers) == 1)

    # Primary objective: minimize cost.
    z = (data["cost"][w][p] * x[w, p] for w, p in product(workers, patients))
    m.setObjectiveN(expr=gp.quicksum(z), index=0, name="cost", priority=1, reltol=alpha)

    # Secondary objective: maximize affinity. Since the model sense is minimize,
    # we negate the secondary objective in order to maximize it.
    w = (data["affinity"][w][p] * x[w, p] for w, p in product(workers, patients))
    m.setObjectiveN(
        expr=gp.quicksum(w), index=1, name="affinity", priority=0, weight=-1
    )

    m.optimize()

    # Extract primary and secondary objective values.
    m.params.ObjNumber = 0
    cost = m.ObjNVal

    m.params.ObjNumber = 1
    affinity = m.ObjNVal

    return cost, affinity


def main():
    data = json.load(sys.stdin)

    out = csv.writer(sys.stdout)
    out.writerow(["alpha", "cost", "affinity"])

    for i in range(21):
        alpha = i / 20.0
        cost, affinity = solve(data, alpha)
        out.writerow([alpha, cost, affinity])


if __name__ == "__main__":
    main()
